using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace EggTimer
{
  public enum Status
  {
    Playing, Paused, Stopped
  }

  public partial class Watch : Form
  {
    private string name;
    private int elapsed, estimated, wasted;
    private Status status;

    public Watch(string name, int estimated)
    {
      InitializeComponent();

      this.name = name;
      this.estimated = estimated;
      elapsed = 0;
      wasted = 0;

      progressBar.Maximum = estimated;
      progressBar.Value = 0;

      estimatedLabel.Text = "Estimated: " + GetTime(estimated, false);

      StopWatch();
    }

    public Status Status
    {
      get { return status; }
    }

    public bool Ended
    {
      get { return elapsed > estimated; }
    }

    private string GetTime(int time, bool remaining)
    {
      int s = time % 60;
      int m = time / 60 % 60;
      int h = time / 3600 % 24;
      int d = time / 86400;

      string days = string.Format("{0:0}", d);
      string hours = string.Format("{0:00}", h);
      string mins = string.Format("{0:00}", m);
      string secs = string.Format("{0:00}", s);

      return (remaining ? "-" : "") + days + ":" + hours + ":" + mins + ":" + secs;
    }

    public bool Tick()
    {
      if (status == Status.Stopped)
        return true;

      if (status == Status.Playing)
        elapsed++;
      else
        wasted++;

      UpdateWatch();

      // true if not ended
      return elapsed < estimated;
    }

    private void EndWatch()
    {
      pauseButton.Enabled = false;
      stopButton.Enabled = false;
      playButton.Enabled = false;

      elapsedLabel.Image = global::EggTimer.Properties.Resources.stop_circle;
      remainingLabel.Image = global::EggTimer.Properties.Resources.stop_circle;
      wastedLabel.Image = global::EggTimer.Properties.Resources.stop_circle;
    }

    public void StopWatch()
    {
      status = Status.Stopped;
      pauseButton.Enabled = false;
      stopButton.Enabled = false;
      playButton.Enabled = true;

      elapsedLabel.Image = global::EggTimer.Properties.Resources.stop_circle;
      remainingLabel.Image = global::EggTimer.Properties.Resources.stop_circle;
      wastedLabel.Image = global::EggTimer.Properties.Resources.stop_circle;

      Text = name + " (stopped)";
    }

    public void PauseWatch()
    {
      status = Status.Paused;
      pauseButton.Enabled = false;
      stopButton.Enabled = true;
      playButton.Enabled = true;

      elapsedLabel.Image = global::EggTimer.Properties.Resources.pause_circle;
      remainingLabel.Image = global::EggTimer.Properties.Resources.pause_circle;
      wastedLabel.Image = global::EggTimer.Properties.Resources.pause_circle;

      tabControl.SelectedTab = wastedPage;
      Text = name + " (paused)";
    }

    public void PlayWatch()
    {
      status = Status.Playing;
      pauseButton.Enabled = true;
      stopButton.Enabled = true;
      playButton.Enabled = false;

      elapsedLabel.Image = global::EggTimer.Properties.Resources.play_circle;
      remainingLabel.Image = global::EggTimer.Properties.Resources.play_circle;
      wastedLabel.Image = global::EggTimer.Properties.Resources.pause_circle;

      if (tabControl.SelectedTab == wastedPage)
        tabControl.SelectedTab = elapsedPage;

      Text = name;
    }

    private void UpdateWatch()
    {
      progressBar.Value = (elapsed < estimated) ? elapsed : estimated;

      elapsedLabel.Text = GetTime(elapsed, false);
      if (elapsed > estimated) elapsedLabel.ForeColor = Color.Red;

      int remaining = (elapsed < estimated) ? (estimated - elapsed) : 0;
      remainingLabel.Text = GetTime(remaining, true);

      wastedLabel.Text = GetTime(wasted, false);
    }

    private void PlayButton(object sender, EventArgs e)
    {
      PlayWatch();
    }

    private void PauseButton(object sender, EventArgs e)
    {
      PauseWatch();
    }

    private void StopButton(object sender, EventArgs e)
    {
      StopWatch();
    }

    private void FormClosingEvent(object sender, FormClosingEventArgs e)
    {
      if (elapsed < estimated)
      {
        DialogResult result = MessageBox.Show(this, "You want to delete this timer?", "Hide or Close",
          MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);

        if (result == DialogResult.Cancel)
        {
          e.Cancel = true;
        }
        else if (result == DialogResult.No)
        {
          e.Cancel = true;
          Hide();
        }
      }
    }



  }
}